/**
 * EasyBeans
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaxXmlWsWebServiceRefVisitor.java 3979 2008-09-28 11:02:27Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.common.impl.configurator.visitor;

import org.ow2.util.ee.metadata.common.api.interfaces.IWebServiceRef;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.impl.struct.JaxwsWebServiceRef;
import org.ow2.util.scan.api.IType;

/**
 * This class manages the handling of &#64;{@link javax.xml.ws.WebServiceRef}
 * annotation.
 * @param <T> An implementation of IWebServiceRef interface.
 * @author Guillaume Sauthier
 */
public class JavaxXmlWsWebServiceRefVisitor<T extends IWebServiceRef> extends AbsAnnotationVisitor<T> {

    /**
     * 'name' attribute of the annotation.
     */
    private static final String ATTR_NAME = "name";

    /**
     * 'mappedName' attribute of the annotation.
     */
    private static final String ATTR_MAPPED_NAME = "mappedName";

    /**
     * 'value' attribute of the annotation.
     */
    private static final String ATTR_VALUE = "value";

    /**
     * 'wsdlLocation' attribute of the annotation.
     */
    private static final String ATTR_WSDL_LOCATION = "wsdlLocation";

    /**
     * 'type' attribute of the annotation.
     */
    private static final String ATTR_TYPE = "type";

    /**
     * Data structure.
     */
    private IJaxwsWebServiceRef webServiceRef = null;

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/xml/ws/WebServiceRef;";

    /**
     * Constructor.
     * @param annotationMetadata linked to a class or method metadata
     */
    public JavaxXmlWsWebServiceRefVisitor(final T annotationMetadata) {
        super(annotationMetadata);
        webServiceRef = new JaxwsWebServiceRef();
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link org.ow2.util.scan.api.IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (name.equals(ATTR_NAME)) {
            webServiceRef.setName((String) value);
        } else if (name.equals(ATTR_TYPE)) {
            IType type = (IType) value;
            webServiceRef.setType(type.getClassName());
        } else if (name.equals(ATTR_MAPPED_NAME)) {
            // mapped-name is not useful for WS
            webServiceRef.setMappedName(((String) value));
        } else if (name.equals(ATTR_VALUE)) {
            IType type = (IType) value;
            webServiceRef.setValue(type.getClassName());
        } else if (name.equals(ATTR_WSDL_LOCATION)) {
            webServiceRef.setWsdlLocation((String) value);
        }
    }

    @Override
    public void visitEnd() {
        getAnnotationMetadata().setJaxwsWebServiceRef(webServiceRef);
    }

    /**
     * @return Internal object used representing &#64;{@link javax.xml.ws.WebServiceRef} annotation.
     */
    protected IJaxwsWebServiceRef getJaxwsWebServiceRef() {
        return this.webServiceRef;
    }

    /**
     * Sets the IJaxwsWebServiceRef object.
     * @param webServiceRef the object which replaced the previous one.
     */
    protected void setJaxwsWebServiceRef(final IJaxwsWebServiceRef webServiceRef) {
        this.webServiceRef = webServiceRef;
    }

    public String getType() {
        return TYPE;
    }
}
